import pytest  # pylint: disable=unused-import
from contigion_indicators.rsi import rsi, rsi_mavg, rsi_over_bought_sold
from contigion_indicators.util.functions import get_dataframe_size
from .setup import data, n_candles

period = 7
mavg = 14


def test_rsi():
    rsi_data = rsi(data, period).drop(columns=['signal']).dropna(inplace=False)
    assert (get_dataframe_size(rsi_data) == (n_candles - period))


def test_rsi_mavg():
    rsi_data = rsi_mavg(data, period, mavg).drop(columns=['signal']).dropna(inplace=False)
    assert (get_dataframe_size(rsi_data) == (n_candles - mavg - period + 1))


def test_rsi_over_bought_sold():
    rsi_data = rsi_over_bought_sold(data, period).drop(columns=['signal']).dropna(inplace=False)
    assert (get_dataframe_size(rsi_data) == (n_candles - period))



import pytest  # pylint: disable=unused-import
from contigion_indicators.parabolic_sar import psar_trend
from contigion_indicators.util.functions import get_dataframe_size
from .setup import data, n_candles


def test_psar_trend():
    sma_data = psar_trend(data)
    assert get_dataframe_size(sma_data) == n_candles



import pytest  # pylint: disable=unused-import
from contigion_indicators.sma import sma_crossover, sma_trend_direction
from contigion_indicators.util.functions import get_dataframe_size
from .setup import data, n_candles

fast = 5
slow = 13
period = 200


def test_sma_crossover():
    sma_data = sma_crossover(data, fast, slow).drop(columns=['signal']).dropna(inplace=False)
    assert (get_dataframe_size(sma_data) == (n_candles - slow + 1))


def test_sma_trend():
    sma_data = sma_trend_direction(data, period).drop(columns=['signal']).dropna(inplace=False)
    assert (get_dataframe_size(sma_data) == (n_candles - period + 1))



import numpy as np
import pandas as pd
from .sma import sma_trend_direction

CANDLESTICK_PATTERNS = ['bull-spinning-top', 'bear-spinning-top', 'bull-marubozu', 'bear-marubozu', 'doji', 'hammer',
                        'hanging-man', 'inverted-hammer', 'shooting-star', 'tweezer-tops', 'tweezer-bottoms',
                        'bullish-harami',
                        'bearish-harami', 'bullish-engulfing', 'bearish-engulfing', 'morning-star', 'evening-star',
                        'three-white-soldiers', 'three-black-crows', 'three-inside-up', 'three-inside-down']


def candle_colour(data):
    result = data.copy(deep=True)

    result['signal'] = None
    result.loc[(result.open < result.close), 'signal'] = 'buy'
    result.loc[(result.open > result.close), 'signal'] = 'sell'

    return result


def candle_size(data, short=0.3, long=0.7):
    result = data.copy(deep=True)

    result['total_candle'] = abs(data['high'] - data['low'])
    result['body'] = abs(data['open'] - data['close'])

    result['signal'] = None
    result.loc[(result['body'] / result['total_candle'] <= short), 'signal'] = 'small'
    result.loc[result['body'] / result['total_candle'] >= long, 'signal'] = 'large'

    # Drop intermediate columns
    result.drop(columns=['body', 'total_candle'], inplace=True)

    return result


def candle_body_info(data, short=0.3, long=0.7):
    result = data.copy(deep=True)

    result['body'] = abs(result['open'] - result['close'])
    result['upper_wick'] = abs(result['high'] - result[['open', 'close']].max(axis=1))
    result['lower_wick'] = abs(result['low'] - result[['open', 'close']].min(axis=1))
    result['total_candle'] = abs(result['high'] - result['low'])
    data['body_middle'] = abs(data['high'] + data['low']) / 2

    result['body_size'] = result['body'] / result['total_candle']
    result['upper_wick_size'] = result['upper_wick'] / result['total_candle']
    result['lower_wick_size'] = result['lower_wick'] / result['total_candle']

    # All relative to a candle's own wick and body size
    result['long_upper_wick'] = result['upper_wick_size'] >= long
    result['short_upper_wick'] = result['upper_wick_size'] <= short
    result['long_lower_wick'] = result['lower_wick_size'] >= long
    result['short_lower_wick'] = result['lower_wick_size'] <= short
    result['long_body'] = result['body_size'] >= long
    result['short_body'] = result['body_size'] <= short

    return result


def multi_candlestick_info(data):
    result = data.copy(deep=True)

    result[['open_1', 'high_1', 'low_1', 'close_1']] = result[['open', 'high', 'low', 'close']].shift(2)
    result[['open_2', 'high_2', 'low_2', 'close_2']] = result[['open', 'high', 'low', 'close']].shift(1)

    result['body_1'] = abs(result['open_1'] - result['close_1'])
    result['body_2'] = abs(result['open_2'] - result['close_2'])
    result['body'] = abs(result['open'] - result['close'])  # candle 3

    result['total_candle_1'] = abs(result['high_1'] - result['low_1'])
    result['total_candle_2'] = abs(result['high_2'] - result['low_2'])
    result['total_candle'] = abs(result['high'] - result['low'])  # candle 3

    result['body_middle_1'] = abs(result['high_1'] + result['low_1']) / 2

    return result


def candlestick_type(data):
    # Candle and body sizes
    result = candle_body_info(data)

    # Trend direction
    result['trend'] = sma_trend_direction(data)['trend']

    # Output
    result['pattern'] = 'unknown'
    result['signal'] = None

    # ----- Bullish Spinning top -----
    bull_spinning_top = np.where((result.body / result.total_candle <= 0.3) &
                                 (result.upper_wick / result.total_candle > 0.2) &
                                 (result.lower_wick / result.total_candle > 0.2) &
                                 (result.trend == 'downtrend') &
                                 (result.pattern == 'unknown'))[0]

    result.loc[bull_spinning_top, 'pattern'] = 'bull-spinning-top'
    result.loc[bull_spinning_top, 'signal'] = 'buy'

    # ----- Bearish Spinning top -----
    bear_spinning_top = np.where((result.body / result.total_candle <= 0.3) &
                                 (result.upper_wick / result.total_candle > 0.2) &
                                 (result.lower_wick / result.total_candle > 0.2) &
                                 (result.trend == 'uptrend') &
                                 (result.pattern == 'unknown'))[0]

    result.loc[bear_spinning_top, 'pattern'] = 'bear-spinning-top'
    result.loc[bear_spinning_top, 'signal'] = 'sell'

    # ----- White Marubozu -----
    white_marubozu = np.where((result.open == result.low) &
                              (result.close == result.high) &
                              (result.pattern == 'unknown'))[0]

    result.loc[white_marubozu, 'pattern'] = 'bull-marubozu'
    result.loc[white_marubozu, 'signal'] = 'buy'

    # ----- Black Marubozu -----
    black_marubozu = np.where((result.open == result.high) &
                              (result.close == result.low) &
                              (result.pattern == 'unknown'))[0]

    result.loc[black_marubozu, 'pattern'] = 'bear-marubozu'
    result.loc[black_marubozu, 'signal'] = 'sell'

    # ----- Doji -----
    doji = np.where((result.open == result.close) &
                    (result.pattern == 'unknown'))[0]

    result.loc[doji, 'pattern'] = 'doji'
    result.loc[doji, 'signal'] = 'sell'

    return result


def single_candlestick_pattern(data):
    # Candle and body sizes
    result = candle_body_info(data)

    # Trend direction
    result['trend'] = sma_trend_direction(result)['trend']

    # Output
    result['pattern'] = 'unknown'
    result['signal'] = None

    # ----- Hammer -----
    hammer = np.where((result[['upper_wick', 'lower_wick']].max(axis=1) > result.body * 2) &
                      (result.upper_wick / result.total_candle < 0.05) &
                      (result[['open', 'close']].min(axis=1) > result.body_middle) &
                      (result.trend == 'downtrend') &
                      (result.pattern == 'unknown'))[0]

    result.loc[hammer, 'pattern'] = 'hammer'
    result.loc[hammer, 'signal'] = 'buy'

    # ----- Hanging man -----
    hanging_man = np.where((result.lower_wick > result.body * 2) &
                           (result.upper_wick / result.total_candle < 0.05) &
                           (result[['open', 'close']].min(axis=1) > result.body_middle) &
                           (result.trend == 'uptrend') &
                           (result.pattern == 'unknown'))[0]

    result.loc[hanging_man, 'pattern'] = 'hanging-man'
    result.loc[hanging_man, 'signal'] = 'sell'

    # ----- Inverted hammer -----
    inverted_hammer = np.where((result.upper_wick > 2 * result.body) &
                               (result.lower_wick / result.total_candle < 0.05) &
                               (result[['open', 'close']].max(axis=1) < result.body_middle) &
                               (result.trend == 'downtrend') &
                               (result.pattern == 'unknown'))[0]

    result.loc[inverted_hammer, 'pattern'] = 'inverted-hammer'
    result.loc[inverted_hammer, 'signal'] = 'buy'

    # ----- Shooting star -----
    shooting_star = np.where((result.upper_wick > 2 * result.body) &
                             (result.lower_wick / result.total_candle < 0.05) &
                             (result[['open', 'close']].max(axis=1) < result.body_middle) &
                             (result.trend == 'uptrend') &
                             (result.pattern == 'unknown'))[0]

    result.loc[shooting_star, 'pattern'] = 'shooting-star'
    result.loc[shooting_star, 'signal'] = 'sell'

    return result


def dual_candlestick_pattern(data):
    # Candle and body sizes
    result = candle_body_info(data)

    # Candlestick 2 and Candlstick 3 result
    result = multi_candlestick_info(data)

    # Trend direction
    result['trend'] = sma_trend_direction(data)['trend']

    # Signal
    result['pattern'] = 'unknown'
    result['signal'] = None

    # ----- Bullish engulfing -----
    bullish_engulfing = np.where((result.trend == 'downtrend') &
                                 (result.open_1 > result.close_1) &
                                 (result.open < result.close) &
                                 (result.close > result.open_1) &
                                 (result.open < result.close_1) &
                                 (result.pattern == 'unknown'))[0]

    result.loc[bullish_engulfing, 'pattern'] = 'bullish-engulfing'
    result.loc[bullish_engulfing, 'signal'] = 'buy'

    # ----- Bearish engulfing -----
    bearish_engulfing = np.where((result.trend == 'uptrend') &
                                 (result.open_1 < result.close_1) &
                                 (result.open > result.close) &
                                 (result.close < result.open_1) &
                                 (result.open > result.close_1) &
                                 (result.pattern == 'unknown'))[0]

    result.loc[bearish_engulfing, 'pattern'] = 'bearish-engulfing'
    result.loc[bearish_engulfing, 'signal'] = 'sell'

    # ----- Bullish harami -----
    bullish_harami = np.where((result.trend == 'downtrend') &
                              (result.open_1 > result.close_1) &
                              (result.open < result.close) &
                              (result.close < result.open_1) &
                              (result.open > result.close_1) &
                              (result.pattern == 'unknown'))[0]

    result.loc[bullish_harami, 'pattern'] = 'bullish-harami'
    result.loc[bullish_harami, 'signal'] = 'buy'

    # ----- Bearish harami -----
    bearish_harami = np.where((result.trend == 'uptrend') &
                              (result.open_1 < result.close_1) &
                              (result.open > result.close) &
                              (result.close > result.open_1) &
                              (result.open < result.close_1) &
                              (result.pattern == 'unknown'))[0]

    result.loc[bearish_harami, 'pattern'] = 'bearish-harami'
    result.loc[bearish_harami, 'signal'] = 'sell'

    # ----- Tweezer tops -----
    tweezer_tops = np.where((result.trend == 'uptrend') &
                            (result.open_1 < result.close_1) &
                            (result.open > result.close) &
                            (result.high_1 == result.high) &
                            (result.pattern == 'unknown'))[0]

    result.loc[tweezer_tops, 'pattern'] = 'tweezer-tops'
    result.loc[tweezer_tops, 'signal'] = 'sell'

    # ----- Tweezer bottoms -----
    tweezer_bottoms = np.where((result.trend == 'downtrend') &
                               (result.open_1 > result.close_1) &
                               (result.open < result.close) &
                               (result.low_1 == result.low) &
                               (result.pattern == 'unknown'))[0]

    result.loc[tweezer_bottoms, 'pattern'] = 'tweezer-bottoms'
    result.loc[tweezer_bottoms, 'signal'] = 'buy'

    return result


def triple_candlestick_pattern(data):
    # Candlestick 2 and Candlstick 3 result
    result = multi_candlestick_info(data)

    # Trend direction
    result['trend'] = sma_trend_direction(result)['trend']

    # Signal
    result['pattern'] = 'unknown'
    result['signal'] = None

    # Candle and body sizes
    multi_candlestick_info(result)

    # ----- Morning star -----
    morning_star = np.where((result.trend == 'downtrend') &
                            (result.body_1 / result.total_candle_1 >= 0.6) &
                            (result.total_candle_2 < result[['total_candle_1', 'total_candle']].max(axis=1) * 0.6) &
                            (result.open_1 > result.close_1) &
                            (result.close > result.body_middle_1) &
                            (result.pattern == 'unknown'))[0]

    result.loc[morning_star, 'pattern'] = 'morning-star'
    result.loc[morning_star, 'signal'] = 'buy'

    # ----- Evening star -----
    evening_star = np.where((result.trend == 'uptrend') &
                            (result.body_1 / result.total_candle_1 >= 0.6) &
                            (result.total_candle_2 < result[['total_candle_1', 'total_candle']].max(axis=1) * 0.6) &
                            (result.open_1 < result.close_1) &
                            (result.close < result.body_middle_1) &
                            (result.pattern == 'unknown'))[0]

    result.loc[evening_star, 'pattern'] = 'evening-star'
    result.loc[evening_star, 'signal'] = 'sell'

    # ----- White soldier -----
    three_white_soldiers = np.where((result.body_2 > result.body_1) &
                                    (result.body / result.total_candle >= 0.5) &
                                    (result.total_candle >= result.body_2) &
                                    (result.open_1 < result.close_1) &
                                    (result.open_2 < result.close_2) &
                                    (result.open < result.close) &
                                    (abs(result.close_2 - result.high_2) / result.total_candle_2 < 0.5) &
                                    (result.pattern == 'unknown'))[0]

    result.loc[three_white_soldiers, 'pattern'] = 'three-white-soldiers'
    result.loc[three_white_soldiers, 'signal'] = 'buy'

    # ----- Black crow -----
    three_black_crows = np.where((result.body_2 > result.body_1) &
                                 (result.body / result.total_candle >= 0.8) &
                                 (result.total_candle >= result.body_2) &
                                 (result.open_1 > result.close_1) &
                                 (result.open_2 > result.close_2) &
                                 (result.open > result.close) &
                                 (abs(result.close_2 - result.low_2) / result.total_candle_2 < 0.3) &
                                 (result.pattern == 'unknown'))[0]

    result.loc[three_black_crows, 'pattern'] = 'three-black-crows'
    result.loc[three_black_crows, 'signal'] = 'sell'

    # ----- Three inside up -----
    three_inside_up = np.where((result.trend == 'downtrend') &
                               (result.open_1 > result.close_1) &
                               (result.close_2 >= result.body_middle_1) &
                               (result.close > result.high_1) &
                               (result.pattern == 'unknown'))[0]

    result.loc[three_inside_up, 'pattern'] = 'three-inside-up'
    result.loc[three_inside_up, 'signal'] = 'buy'

    # ----- Three inside down -----
    three_inside_down = np.where((result.trend == 'uptrend') &
                                 (result.open_1 < result.close_1) &
                                 (result.close_2 <= result.body_middle_1) &
                                 (result.close < result.low_1) &
                                 (result.pattern == 'unknown'))[0]

    result.loc[three_inside_down, 'pattern'] = 'three-inside-down'
    result.loc[three_inside_down, 'signal'] = 'sell'

    return result


def ml_candle_colour(data, column_prefix="candle_colour"):
    candle_data = candle_colour(data)
    result = pd.get_dummies(candle_data['signal'], dtype=int, prefix=column_prefix)

    return result


def ml_candle_size(data, short=0.3, long=0.7, column_prefix="candle_size"):
    candle_data = candle_size(data, short, long)
    result = pd.get_dummies(candle_data['signal'], dtype=int, prefix=column_prefix)

    return result


def ml_candlestick_pattern(data):
    candlestick_types = get_candlestick_type(candlestick_type(data))
    single_candlesticks = get_candlestick_type(single_candlestick_pattern(data))
    dual_candlesticks = get_candlestick_type(dual_candlestick_pattern(data))
    triple_candlesticks = get_candlestick_type(triple_candlestick_pattern(data))
    result = pd.concat([candlestick_types, single_candlesticks, dual_candlesticks, triple_candlesticks], axis=1)
    result.drop(columns=['unknown'], inplace=True)

    missing_columns = list(set(CANDLESTICK_PATTERNS) - set(result.columns))
    result[missing_columns] = 0

    return result


def get_candlestick_type(data):
    result = pd.get_dummies(data['pattern'], dtype=int)
    return result



import pytest  # pylint: disable=unused-import
from contigion_indicators.macd import macd_crossover
from contigion_indicators.util.functions import get_dataframe_size
from .setup import data, n_candles

fast = 12
slow = 26
signal = 9


def test_maccd_crossover():
    macd_data = macd_crossover(data, fast, slow, signal).drop(columns=['signal']).dropna(inplace=False)
    assert (get_dataframe_size(macd_data) == (n_candles - slow - signal + 2))



import pandas_ta as ta  # pylint: disable=unused-import
from .util.functions import validate_input, validate_output


def get_psar_data(data):
    required_columns = ['close']
    validate_input(data, required_columns, [])

    result = data.copy(deep=True)
    psar = result.ta.psar()
    result['psar_up'] = psar['PSARs_0.02_0.2']
    result['psar_down'] = psar['PSARl_0.02_0.2']

    validate_output(result)
    return result


def psar_trend(data):
    result = get_psar_data(data)

    # Generate buy/sell signals
    result['signal'] = None
    result.loc[(result['psar_up'].isnull()), 'signal'] = 'buy'
    result.loc[(result['psar_down'].isnull()), 'signal'] = 'sell'

    validate_output(result)

    return result



import pytest  # pylint: disable=unused-import
from contigion_indicators.supertrend import supertrend, supertrend_direction
from contigion_indicators.util.functions import get_dataframe_size
from .setup import data, n_candles

atr_length = 7
multiplier = 3
offset = 0


def test_supertrend():
    supertrend_data = supertrend(data, atr_length, multiplier, offset).dropna(inplace=False)
    assert (get_dataframe_size(supertrend_data) == (n_candles - atr_length))


def test_supertrend_direction():
    supertrend_data = supertrend_direction(data, atr_length, multiplier, offset).drop(columns=['signal']).dropna(
        inplace=False)
    assert (get_dataframe_size(supertrend_data) == (n_candles - atr_length))



from datetime import time
import numpy as np

# Sessions
# USES SA TIME
# Source: https://www.babypips.com/learn/forex/forex-trading-sessions
# and https://www.xm.com/education/chapter-1/trading-sessions
SYDNEY_OPEN = time(0, 00)
SYDNEY_CLOSE = time(8, 00)
TOKYO_OPEN = time(2, 00)
TOKYO_CLOSE = time(10, 00)
LONDON_OPEN = time(9, 00)
LONDON_CLOSE = time(18, 00)
NEW_YORK_OPEN = time(15, 00)
NEW_YORK_CLOSE = time(23, 00)

SESSIONS = ['sydney_session', 'tokyo_session', 'london_session', 'new_york_session']
WEEKDAYS = ['monday', 'tuesday', 'wednesday', 'thursday', 'friday']


def trading_session(data):
    result = data.copy(deep=True)
    result['hour'] = result['time'].dt.time

    result[SESSIONS] = 0
    result.loc[(result.hour > SYDNEY_OPEN) & (result.hour < SYDNEY_CLOSE), 'sydney_session'] = 1
    result.loc[(result.hour > TOKYO_OPEN) & (result.hour < TOKYO_CLOSE), 'tokyo_session'] = 1
    result.loc[(result.hour > LONDON_OPEN) & (result.hour < LONDON_CLOSE), 'london_session'] = 1
    result.loc[(result.hour > NEW_YORK_OPEN) & (result.hour < NEW_YORK_CLOSE), 'new_york_session'] = 1

    return result


def day_of_the_week(data):
    result = data.copy(deep=True)
    result['weekday'] = result['time'].dt.day_name()

    result[WEEKDAYS] = 0
    result['monday'] = np.where(result.weekday == 'Monday', 1, 0)
    result['tuesday'] = np.where(result.weekday == 'Tuesday', 1, 0)
    result['wednesday'] = np.where(result.weekday == 'Wednesday', 1, 0)
    result['thursday'] = np.where(result.weekday == 'Thursday', 1, 0)
    result['friday'] = np.where(result.weekday == 'Friday', 1, 0)

    result.drop(columns=['weekday'], inplace=True)

    return result



import pandas_ta as ta  # pylint: disable=unused-import
from .util.functions import validate_input, validate_output


def get_sma_data(data, fast=1, slow=1, period=1):
    required_columns = ['close']
    periods = [fast, slow, period]
    validate_input(data, required_columns, periods)

    result = data.copy(deep=True)
    result['sma'] = result.ta.sma(period)
    result['sma_slow'] = result.ta.sma(slow)
    result['sma_fast'] = result.ta.sma(fast)

    validate_output(result)
    return result


def sma_crossover(data, fast=5, slow=13):
    result = get_sma_data(data, fast, slow)
    result['prev_slow'] = result['sma_slow'].shift(1)
    result['prev_fast'] = result['sma_fast'].shift(1)

    # Generate buy/sell signals
    sma_zip = zip(result['sma_slow'], result['sma_fast'], result['prev_slow'], result['prev_fast'])
    result['signal'] = [
        'buy' if (curr_slow > curr_fast) and (prev_slow < prev_fast) else
        'sell' if (curr_slow < curr_fast) and (prev_slow > prev_fast) else
        None
        for curr_slow, curr_fast, prev_slow, prev_fast in sma_zip
    ]

    # Drop intermediate columns
    result.drop(columns=['prev_slow', 'prev_fast', 'sma'], inplace=True)
    validate_output(result)

    return result


def sma_trend_direction(data, period=200):
    result = get_sma_data(data, period=period)

    # Generate buy/sell signals
    result['trend_direction'] = None
    result.loc[(result.close > result.sma), 'trend_direction'] = 'buy'
    result.loc[(result.close < result.sma), 'trend_direction'] = 'sell'

    result['signal'] = result['trend_direction'].shift(1)

    # Drop intermediate columns
    result.drop(columns=['trend_direction', 'sma_slow', 'sma_fast'], inplace=True)
    validate_output(result)

    return result



import pandas_ta as ta  # pylint: disable=unused-import
from .util.functions import validate_input, validate_output


def get_bb_data(data, period, std_dev):
    required_columns = ['close']
    periods = [period]
    validate_input(data, required_columns, periods)

    result = data.copy(deep=True)
    bbands = result.ta.bbands(length=period, std=std_dev)

    # Assign the Bollinger Bands to the result DataFrame
    result['lower'] = bbands[f"BBL_{period}_{std_dev}.0"]
    result['upper'] = bbands[f"BBU_{period}_{std_dev}.0"]
    result['mavg'] = bbands[f"BBM_{period}_{std_dev}.0"]

    validate_output(result)

    return result


def bollinger_bands(data, period=5, std_dev=2):
    result = get_bb_data(data, period, std_dev)

    # Generate buy / sell signals
    close_mavg = zip(result['close'], result['mavg'])
    result['signal'] = [
        'buy' if (close > mavg) else
        'sell' if (close < mavg) else
        None
        for close, mavg in close_mavg
    ]

    validate_output(result)

    return result



import pytest  # pylint: disable=unused-import
from contigion_indicators.bollinger_bands import bollinger_bands
from contigion_indicators.util.functions import get_dataframe_size
from .setup import data, n_candles

period = 7


def test_bollinger_bands():
    bb_data = bollinger_bands(data, period).drop(columns=['signal']).dropna(inplace=False)
    assert (get_dataframe_size(bb_data) == (n_candles - period + 1))



import pandas_ta as ta  # pylint: disable=unused-import
from .util.functions import validate_input, validate_output


def get_supertrend_data(data, atr_length, multiplier, offset):
    required_columns = ['close']
    periods = [atr_length, multiplier, offset]
    validate_input(data, required_columns, periods)

    result = data.copy(deep=True)
    supertrend_column = f"SUPERT_{atr_length}_{multiplier}.{offset}"
    supertrend_data = result.ta.supertrend(atr_length, multiplier, offset)

    result['supertrend'] = supertrend_data[supertrend_column]
    result.loc[0, 'supertrend'] = None

    validate_output(result)
    return result


def supertrend(data, atr_length=7, multiplier=3, offset=0):
    result = get_supertrend_data(data, atr_length, multiplier, offset)
    validate_output(result)

    return result


def supertrend_direction(data, atr_length=7, multiplier=3, offset=0):
    result = get_supertrend_data(data, atr_length, multiplier, offset)

    # Generate buy/sell signals
    result['signal'] = None
    result.loc[(result['close'] > result['supertrend']), 'signal'] = 'buy'
    result.loc[(result['close'] < result['supertrend']), 'signal'] = 'sell'

    validate_output(result)

    return result



import pandas_ta as ta  # pylint: disable=unused-import
from .util.functions import validate_input, validate_output


def get_macd_data(data, fast, slow, signal):
    required_columns = ['close']
    periods = [slow + signal]
    validate_input(data, required_columns, periods)

    result = data.copy(deep=True)
    result[['macd', 'histogram', 'signal_line']] = result.ta.macd(fast=fast, slow=slow, signal=signal)
    result['prev_signal'] = result['signal_line'].shift(1)
    result['prev_macd'] = result['macd'].shift(1)

    validate_output(result)

    return result


def macd_crossover(data, fast=12, slow=26, signal=9):
    result = get_macd_data(data, fast, slow, signal)

    # Generate buy/sell signals
    macd_zip_pairs = zip(result['macd'], result['signal_line'], result['histogram'],
                         result['prev_macd'], result['prev_signal'])

    result['signal'] = [
        'buy' if signal_line < macd < histogram and prev_signal > prev_macd else
        'sell' if histogram < macd < signal_line and prev_signal < prev_macd else
        None
        for macd, signal_line, histogram, prev_macd, prev_signal in macd_zip_pairs
    ]

    # Drop intermediate columns
    result.drop(columns=['prev_signal', 'prev_macd'], inplace=True)
    validate_output(result)

    return result



import pandas_ta as ta  # pylint: disable=unused-import
from .util.functions import validate_input, validate_output


def get_rsi_data(data, period, mavg=14):
    required_columns = ['close']
    periods = [period, mavg]
    validate_input(data, required_columns, periods)

    result = data.copy(deep=True)
    result['rsi'] = result.ta.rsi(period)

    validate_output(result)
    return result


def rsi(data, period=7, overbought=70, oversold=30):
    result = get_rsi_data(data, period)
    result['prev_rsi'] = result['rsi'].shift(1)

    # Generate buy/sell signals
    rsi_zip = zip(result['rsi'], result['prev_rsi'])

    result['signal'] = [
        'buy' if (curr_rsi > oversold > prev_rsi) else
        'sell' if (curr_rsi < overbought < prev_rsi) else
        None
        for curr_rsi, prev_rsi in rsi_zip
    ]

    # Drop intermediate columns
    result.drop(columns=['prev_rsi'], inplace=True)
    validate_output(result)

    return result


def rsi_mavg(data, period=7, mavg=14):
    result = get_rsi_data(data, period, mavg)
    result['mavg'] = result['rsi'].rolling(mavg).mean()
    result['prev_mavg'] = result['mavg'].shift(1)
    result['prev_rsi'] = result['rsi'].shift(1)

    # Generate buy/sell signals
    rsi_mavg_zip = zip(result['rsi'], result['prev_rsi'], result['mavg'], result['prev_mavg'])

    result['signal'] = [
        'buy' if (50 > curr_rsi > curr_mavg) and (prev_rsi < prev_mavg) else
        'sell' if (50 < curr_rsi < curr_mavg) and (prev_rsi > prev_mavg) else
        None
        for curr_rsi, prev_rsi, curr_mavg, prev_mavg in rsi_mavg_zip
    ]

    # Drop intermediate columns
    result.drop(columns=['prev_rsi', 'prev_mavg'], inplace=True)
    validate_output(result)

    return result


def rsi_over_bought_sold(data, period=7, overbought=70, oversold=30):
    result = get_rsi_data(data, period)

    # Generate buy/sell signals
    result['signal'] = None
    result.loc[(result.rsi < oversold), 'signal'] = 'buy'
    result.loc[(result.rsi > overbought), 'signal'] = 'sell'

    validate_output(result)

    return result



__all__ = ["bollinger_bands", "candlestick_patterns", "macd", "parabolic_sar", "rsi", "sessions", "sma", "supertrend",
           "support_and_resistance"]

from . import util
from .bollinger_bands import bollinger_bands
from .candlestick_patterns import candle_colour, candle_body_info, multi_candlestick_info, triple_candlestick_pattern
from .candlestick_patterns import candlestick_type, single_candlestick_pattern, dual_candlestick_pattern, candle_size
from .candlestick_patterns import ml_candle_colour, ml_candle_size, ml_candlestick_pattern, get_candlestick_type
from .macd import macd_crossover
from .parabolic_sar import psar_trend
from .rsi import rsi, rsi_mavg, rsi_over_bought_sold
from .sessions import trading_session, day_of_the_week
from .supertrend import supertrend, supertrend_direction
from .support_and_resistance import support_and_resistance, get_support_and_resistance_levels



import numpy as np
from .util.metatrader import get_point


def support_and_resistance(symbol, data, window=2, target=10):
    point = get_point(symbol)
    s_and_r, support, resistance = get_support_and_resistance_levels(data, window)
    close = s_and_r['close'].values
    suport_levels = np.array(support['level'])
    resistance_levels = np.array(resistance['level'])

    s_and_r['is_support_close'] = np.any(abs(close[:, np.newaxis] - suport_levels) < (target * point), axis=1)
    s_and_r['is_resistance_close'] = np.any(abs(close[:, np.newaxis] - resistance_levels) < (target * point), axis=1)

    return s_and_r


def get_support_and_resistance_levels(data, window=5):
    result = data.copy(deep=True)

    # Get candles within window
    for i in range(1, window + 1):
        result[f"previous_candle_high_{i}"] = result['high'].shift(i)
        result[f"previous_candle_low_{i}"] = result['low'].shift(i)
        result[f"next_candle_high_{i}"] = result['high'].shift(-i)
        result[f"next_candle_low_{i}"] = result['low'].shift(-i)

    # Get valid support indices / levels
    result['is_support'] = (result
                            .filter(regex="previous_candle_low_|next_candle_low_")
                            .ge(result['low'], axis=0)
                            .all(axis=1))

    # Get valid resisitance indices / levels
    result['is_resistance'] = (result
                               .filter(regex="previous_candle_high_|next_candle_high_")
                               .le(result['high'], axis=0)
                               .all(axis=1))

    support = result[result['is_support']].copy(deep=True)
    resistance = result[result['is_resistance']].copy(deep=True)

    broken_support_indices = []
    broken_resistance_indices = []

    # Remove violated levels
    for index, row in support.iterrows():
        next_index = index + 1
        is_level_broken = np.any(result.filter(items=['close']).loc[next_index:] < row.low)

        if is_level_broken:
            broken_support_indices.append(index)

    for index, row in resistance.iterrows():
        next_index = index + 1
        is_level_broken = np.any(result.filter(items=['close']).loc[next_index:] > row.low)

        if is_level_broken:
            broken_resistance_indices.append(index)

    support.drop(broken_support_indices, inplace=True)
    resistance.drop(broken_resistance_indices, inplace=True)
    support['level'] = support['low']
    resistance['level'] = resistance['high']

    # Drop columns
    added_columns = result.filter(regex='previous_candle_low_|next_candle_low_|previous_candle_high_|next_candle_high_')
    result.drop(columns=added_columns.columns, inplace=True)

    return result, support[['time', 'level']], resistance[['time', 'level']]
